#!/system/bin/sh

iptables=/system/bin/iptables
ss_dir=${ss_dir}

killprocess() {
    sl=`pidof -s ss-redir`
    st=`pidof -s ss-tunnel`
    if [ -n "$sl" ]; then
        kill -9 $sl
    fi
    if [ -n "$st" ]; then
        kill -9 $st
    fi
}

checknumber() {
    case $1 in
        ''|*[!0-9]*)
            echo "arg '$1' is not valid number."
            exit 1
            ;;
        *) ;;
    esac
}

startfw() {
    killprocess
    $iptables -t nat -N gfw_nat_out 2>/dev/null
    $iptables -t nat -F gfw_nat_out 2>/dev/null
    $iptables -t nat -D OUTPUT -j gfw_nat_out 2>/dev/null
    $iptables -t nat -I OUTPUT -j gfw_nat_out
    $iptables -t nat -A gfw_nat_out -m state --state NEW -p udp --dport 53 -j DNAT --to-destination 127.0.0.1:5533

    $ss_dir/ss-redir -b 127.0.0.1 -u -c $ss_dir/ss-client.json -f $ss_dir/ss-redir.pid
    $ss_dir/ss-tunnel -b 127.0.0.1 -c $ss_dir/ss-tunnel.json -L "8.8.8.8:53" -u -f $ss_dir/ss-server.pid
}

stopfw() {
    $iptables -t nat -D OUTPUT -j gfw_nat_out 2>/dev/null
    $iptables -t nat -F gfw_nat_out 2>/dev/null
    $iptables -t nat -X gfw_nat_out 2>/dev/null

    killprocess
    $iptables -t nat -D OUTPUT -m state --state NEW -p udp --dport 53 -j DNAT --to-destination 127.0.0.1:5533 2>/dev/null
    $iptables -t nat -D OUTPUT -m state --state NEW -p tcp --dport 80 -j DNAT --to-destination 127.0.0.1:1080 2>/dev/null
    $iptables -t nat -D OUTPUT -m state --state NEW -p tcp --dport 443 -j DNAT --to-destination 127.0.0.1:1080 2>/dev/null
}

all() {
    flushrule
    $iptables -t nat -A gfw_nat_out -m state --state NEW -d 192.168.100.1/24 -p tcp -j ACCEPT
    $iptables -t nat -A gfw_nat_out -m state --state NEW -p tcp -j DNAT --to-destination 127.0.0.1:1080
}

addrule() {
    if [ "$#" -ne 1 ]; then
        echo "Illegal number of parameters"
        echo "Usage: ss addrule 10002"
        exit 1
    fi
    checknumber $1
    appid=$1

    $iptables -t nat -D gfw_nat_out -m state --state NEW -p tcp -m owner --uid-owner $appid -j DNAT --to-destination 127.0.0.1:1080 2>/dev/null
    $iptables -t nat -A gfw_nat_out -m state --state NEW -p tcp -m owner --uid-owner $appid -j DNAT --to-destination 127.0.0.1:1080
}

delrule() {
    if [ "$#" -ne 1 ]; then
        echo "Illegal number of parameters"
        echo "Usage: ss delrule 10002"
        exit 1
    fi
    checknumber $1
    appid=$1
    $iptables -t nat -D gfw_nat_out -m state --state NEW -p tcp -m owner --uid-owner $appid -j DNAT --to-destination 127.0.0.1:1080 2>/dev/null
}

flushrule() {
    $iptables -t nat -F gfw_nat_out 2>/dev/null
    $iptables -t nat -A gfw_nat_out -m state --state NEW -p udp --dport 53 -j DNAT --to-destination 127.0.0.1:5533
}

case "$1" in
    start)
        startfw
        ;;
    stop)
        stopfw
        ;;
    all)
        all
        ;;
    addrule)
        addrule $2
        ;;
    delrule)
        delrule $2
        ;;
    flushrule)
        flushrule
        ;;
    *)
        echo "Usage: ss {start|stop|all|addrule|delrule|flushrule}"
        exit 1
esac
